;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2003 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

;------------------------------------------------------------------------------
;
; Host communications implementation
; Communication with the ColdFire has the same interface as in the monosynth 
; application
;
;------------------------------------------------------------------------------

HOST_BUFFER_SIZE	equ	64
HOST_BUFFER_SIZE_MASK	equ	(HOST_BUFFER_SIZE-1)

;------------------------------------------------------------------------------

	org	Xl:

ptrHostTxDataWrite:
		dc	bufferHostTx
ptrHostTxDataRead:
		dc	bufferHostTx

ptrHostRxDataWrite:
		dc	bufferHostRx
ptrHostRxDataRead:
		dc	bufferHostRx

nUnreadCount:
		dc	0
nProcessedCount:
		dc	0

nBlockCounter:
		dc	0
nUsage:
		dc	0

;------------------------------------------------------------------------------

	org	Yl:
	
stackHostRx:	ds	8			; Saves (R7,X0)(R0,M0)(A1,A0)(A2) in the Host RX interrupt
stackHostTx:	ds	8			; Saves (R7,X0)(R0,M0)(A1,A0)(A2) in the Host TX interrupt

;------------------------------------------------------------------------------
	
	org	Xh:

bufferHostTx:	dsm	HOST_BUFFER_SIZE	; Buffer for host data to be transmitted
bufferHostRx:	dsm	HOST_BUFFER_SIZE	; Buffer for host data received

;------------------------------------------------------------------------------

	org	Ph:

;------------------------------------------------------------------------------
ConfigureHostInterrupts:
;------------------------------------------------------------------------------
	;**********************************************************************
	; Disable Host Port
	;*********************************************************************/
	BCLR	#HPCR_HEN,X:<<HPCR

	;**********************************************************************
	;* Disable Host Interrupts
	;*********************************************************************/
	BCLR 	#HCR_HRIE,X:<<HCR
	BCLR 	#HCR_HTIE,X:<<HCR
	BCLR 	#HCR_HCIE,X:<<HCR
	 
	;**********************************************************************
	; Initialize private data
	;*********************************************************************/

	MOVE	#>bufferHostRx,X0
	MOVE	X0,X:ptrHostRxDataWrite
	MOVE	X0,X:ptrHostRxDataRead
	
	MOVE	#>bufferHostTx,X0
	MOVE	X0,X:ptrHostTxDataWrite
	MOVE	X0,X:ptrHostTxDataRead
	
	;**********************************************************************
	; Enable Host Interrupts
	;*********************************************************************/
	BSET	#HCR_HRIE,X:<<HCR

	;**********************************************************************
	; Enable Host Port
	;*********************************************************************/
	BSET	#HPCR_HEN,X:<<HPCR

	RTS


	org	Pl:

;------------------------------------------------------------------------------
interruptHostRxDataFull:
;------------------------------------------------------------------------------
	MOVE	R7,Y:stackHostRx
	MOVE	#stackHostRx+1,R7
	NOP
	NOP
	NOP
	MOVE	X0,Y:(R7)+
	MOVE	R0,Y:(R7)+
	MOVE	M0,Y:(R7)+
	MOVE	X:ptrHostRxDataWrite,R0
	MOVE	#HOST_BUFFER_SIZE_MASK,M0
	MOVE	A0,Y:(R7)+
	MOVE	A1,Y:(R7)+
	MOVE	A2,Y:(R7)

	MOVE	X:ptrHostRxDataRead,X0
	MOVEP	X:<<HRX,X:(R0)+
	MOVE	(R0)+
	MOVE	R0,A
	CMP	X0,A			(R0)-
	BNE	_L
	BCLR	#HCR_HRIE,X:<<HCR
_L:
	MOVE	R0,X:ptrHostRxDataWrite
	MOVE	Y:(R7)-,A2
	MOVE	Y:(R7)-,A1
	MOVE	Y:(R7)-,A0
	MOVE	Y:(R7)-,M0
	MOVE	Y:(R7)-,R0
	MOVE	Y:(R7)-,X0
	MOVE	Y:(R7),R7
	RTI


;------------------------------------------------------------------------------
interruptHostTxDataEmpty:
;------------------------------------------------------------------------------
	MOVE	R7,Y:stackHostTx
	MOVE	#stackHostTx+1,R7
	NOP
	NOP
	NOP
	MOVE	X0,Y:(R7)+
	MOVE	R0,Y:(R7)+
	MOVE	M0,Y:(R7)+
	MOVE	X:ptrHostTxDataRead,R0
	MOVE	#HOST_BUFFER_SIZE_MASK,M0
	MOVE	A0,Y:(R7)+
	MOVE	A1,Y:(R7)+
	MOVE	A2,Y:(R7)

	MOVE	X:ptrHostTxDataWrite,X0
	MOVEP	X:(R0)+,X:<<HTX
	MOVE	R0,A
	CMP	X0,A
	BNE	_L
	BCLR	#HCR_HTIE,X:<<HCR
_L:
	MOVE	R0,X:ptrHostTxDataRead
	MOVE	Y:(R7)-,A2
	MOVE	Y:(R7)-,A1
	MOVE	Y:(R7)-,A0
	MOVE	Y:(R7)-,M0
	MOVE	Y:(R7)-,R0
	MOVE	Y:(R7)-,X0
	MOVE	Y:(R7),R7
	RTI


	org	Pl:

;------------------------------------------------------------------------------
ProcessHostData:
;------------------------------------------------------------------------------
	MOVE	#>32,X0
	MOVE	X0,X:nProcessedCount
_loopCheck:
; 	GetHostReceivedDataCount()
	MOVE	X:ptrHostRxDataWrite,A
	MOVE	X:ptrHostRxDataRead,X0
	SUB	X0,A		X0,R7
	BEQ	_endloopCheck
	MOVE	#HOST_BUFFER_SIZE_MASK,M7
	BGT	_unreadCountPositive
	ADD	#>HOST_BUFFER_SIZE,A
_unreadCountPositive:
;	header = *ptrHostRxDataRead
	MOVE	X:(R7)+,B
	MOVE	A,X:nUnreadCount
;	header.nDevice
	EXTRACTU #>$6022,B,A
	NOP
	MOVE	A0,Y0
;	header.nParameter
	EXTRACTU #>$A018,B,A
	MOVE	#tableCommandHandlers,R1
	MOVE	A0,N1
;	header.nCount
	EXTRACTU #>$8028,B,A
	MOVE	X:nUnreadCount,X0
	MOVEM	P:(R1+N1),R4
	TST	A		A0,B
	BEQ	_messageCountZero
	CMP	X0,B
	BGE	_endloopCheck
	ADD	#>1,B
	BRA	_next

_messageCountZero:
	MOVE	#>1,B
_next:
;	nParameter
	MOVE	N1,A
	MOVE	B,X:nUnreadCount
;	nDevice
	TFR	Y0,B
	JSR	R4
	MOVE	X:ptrHostRxDataRead,R7
	MOVE	X:nUnreadCount,N7
	MOVE	X:nProcessedCount,A
	MOVE	N7,X0
	NOP
	SUB	X0,A		(R7)+N7
	MOVE	R7,X:ptrHostRxDataRead
	BSET	#HCR_HRIE,X:<<HCR
	BLT	_endloopCheck
	MOVE	A,X:nProcessedCount
	BRA	_loopCheck
	
_endloopCheck:
	MOVE	#$FFFFFF,M7
	RTS


	org	Pl:
	
;------------------------------------------------------------------------------
WriteHostData:
;		input R0 = X:ptrData
;		input A = nDataSize
;------------------------------------------------------------------------------
	MOVE	#HOST_BUFFER_SIZE_MASK,M1
	MOVE	X:ptrHostTxDataWrite,R1
	MOVE	M1,M4
	NOP
	NOP
	LUA	(R1)+,R4
	
	.LOOP	A
	;----------------------------------------------------------------------
	  MOVE	R4,Y0
_testfull:
	  MOVE	X:ptrHostTxDataRead,B
	  CMP	Y0,B
	  BNE	_nofull
	  BSET	#HCR_HTIE,X:<<HCR
	  BRA	_testfull
_nofull:
	  MOVE	X:(R0)+,X1
	  MOVE	X1,X:(R1)+	Y:(R4)+,Y0
	;----------------------------------------------------------------------
	.ENDL

	MOVE	R1,X:ptrHostTxDataWrite
	BSET	#HCR_HTIE,X:<<HCR

	MOVE	M7,M1
	MOVE	M7,M4
	RTS


	org	Ph:

;------------------------------------------------------------------------------
WriteHostText:
;		input R0 = X:textData (NULL terminated)
;------------------------------------------------------------------------------
	MOVE	R0,R1
	MOVE	#0,R4
	NOP
	NOP
_loop:
	MOVE	X:(R1),A
	TST	A
	BEQ	_end
	ORI	#$000380,A
	MOVE	A,X:(R1)+	Y:(R4)+,Y0
	BRA	_loop

_end:
	MOVE	R4,A
	BSR	WriteHostData
	RTS	


	org	Ph:

;------------------------------------------------------------------------------
WriteHostDspUsage:
;------------------------------------------------------------------------------
	MOVE	Y:ActiveBufferDAC,B
	MOVE	#>bufferDAC1,Y0
	MOVE	#>bufferDAC2,Y1
	MOVEP	X:<<DSR0,A
	CMP	Y0,B
	BEQ	_getUsageDAC2
_getUsageDAC1:
	SUB	Y0,A
	BRA	_next

_getUsageDAC2:
	SUB	Y1,A

_next:
	MOVE	X:nUsage,X0
	CMP	X0,A
	TLT	X0,A
	
	MOVE	X:nBlockCounter,B
	NOP
	BRCLR	#10,B,_end

	MOVE	#>((100.0/256)/(2*DMA_BLOCK_SIZE)),X0
	MOVE	A,X1
	MPY	X1,X0,B		#>127,X0
	ASL	#8,B,B
	RND	B
	CMP	X0,B
	TGT	X0,B
	
	ANDI	#>$00007F,B
	ORI	#>$000300,B
	MOVE	#nUsage,R0
	MOVE	B,X:nUsage
	MOVE	#>1,A
	BSR	WriteHostData

	CLR	B
	CLR	A
_end:
	ADD	#>1,B
	MOVE	A,X:nUsage
	MOVE	B,X:nBlockCounter
	RTS

;------------------------------------------------------------------------------

	org	Pl:

tableCommandHandlers:
	dc	CommandWriteBlockP
	dc	CommandWriteBlockX
	dc	CommandWriteBlockY
	dc	CommandWriteWordP
	dc	CommandWriteWordX
	dc	CommandWriteWordY
	dc	CommandCopyBlockP
	dc	CommandFillBlockX
	dc	CommandFillBlockY


;------------------------------------------------------------------------------
CommandWriteBlockP:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = len
	DOR	X0,CommandWriteBlockP_Loop
	  MOVE	X:(R7)+,X1
	  MOVEM	X1,P:(R4)+
	  NOP
CommandWriteBlockP_Loop:
	RTS


;------------------------------------------------------------------------------
CommandWriteBlockX:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = len
	DOR	X0,CommandWriteBlockX_Loop
	  MOVE	X:(R7)+,X1
	  MOVE	X1,X:(R4)+
CommandWriteBlockX_Loop:
	RTS


;------------------------------------------------------------------------------
CommandWriteBlockY:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = len
	DOR	X0,CommandWriteBlockY_Loop
	  MOVE	X:(R7)+,X1
	  MOVE	X1,Y:(R4)+
CommandWriteBlockY_Loop:
	RTS


;------------------------------------------------------------------------------
CommandWriteWordP:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = value
	NOP
	NOP
	MOVEM	X0,P:(R4)
	RTS


;------------------------------------------------------------------------------
CommandWriteWordX:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = value
	NOP
	NOP
	MOVE	X0,X:(R4)
	RTS


;------------------------------------------------------------------------------
CommandWriteWordY:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = value
	NOP
	NOP
	MOVE	X0,Y:(R4)
	RTS


;------------------------------------------------------------------------------
CommandCopyBlockP:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = source addr
	MOVE	X:(R7)+,R5			; R5 = target addr
	MOVE	X:(R7)+,X0			; X0 = len
	DOR	X0,CommandCopyBlockP_Loop
	  MOVEM P:(R4)+,X1
	  MOVEM	X1,P:(R5)+
	  NOP
CommandCopyBlockP_Loop:
	RTS


;------------------------------------------------------------------------------
CommandFillBlockX:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = len
	MOVE	X:(R7)+,A			; A  = value
	DOR	X0,CommandFillBlockX_Loop
	  MOVE	A,X:(R4)+
CommandFillBlockX_Loop:
	RTS


;------------------------------------------------------------------------------
CommandFillBlockY:
	; R7 = buffer
;------------------------------------------------------------------------------
	MOVE	X:(R7)+,R4			; R4 = addr
	MOVE	X:(R7)+,X0			; X0 = len
	MOVE	X:(R7)+,A			; A  = value
	DOR	X0,CommandFillBlockY_Loop
	  MOVE	A,Y:(R4)+
CommandFillBlockY_Loop:
	RTS

