;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2003 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD,NOPS
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	list

;-------------------------------------------------------------------------------
; Interrupt vectors
;-------------------------------------------------------------------------------

	org	P:VecHwReset
	BRA	>Start

	org	P:VecHostRxDataFull
	JSR	>interruptHostRxDataFull

	org	P:VecHostTxDataEmpty
	JSR	>interruptHostTxDataEmpty

;-------------------------------------------------------------------------------
; Memory organisation
;-------------------------------------------------------------------------------
	
	org	Xl:$000000	; internal X memory
	org	Yl:$000000	; internal Y memory
	
	org	Pl:$000100	; internal P memory
	org	Ph:$400000	; internal P memory
	
	org	Xh:$440000	; external X memory (with an arbitrary offset of $40000 
				; to avoid overlapping with the external P memory)
	
;-------------------------------------------------------------------------------

	include 'resample_data.inc'
	
;-------------------------------------------------------------------------------

	;-----------------------------------------------------------------------		
	;
	; Adjust BLOCK_SIZE to your needs with the following restictions:
	;
	; 	- DMA_BLOCK_SIZE has to be divisible by both kM and kL
	;	- DMA_BLOCK_SIZE should be less than 4095 due to the loops 
	;	  implementation in the example
	;.
	;-----------------------------------------------------------------------

BLOCK_SIZE		equ	64
DMA_BLOCK_SIZE		equ	BLOCK_SIZE*kM/kL

;-------------------------------------------------------------------------------

	org	Yl:
	
ActiveBufferDAC dc	bufferDAC2		; Current output buffer to be used
ActiveBufferADC	dc	bufferADC2		; Current input buffer to be used

;-------------------------------------------------------------------------------
; Application variables
	
MasterVolume:	
		dc	$000000
MasterVolumeTarget:	
		dc	$000000

BypassEnabled:
		dc	$000000
BypassEnabledTarget:
		dc	$000000

;-------------------------------------------------------------------------------
	
	org	Xh:

bufferDAC:
bufferDAC1	ds	2*DMA_BLOCK_SIZE	; Buffer1 for stereo audio to be transmitted (at 48kHz)
bufferDAC2	ds	2*DMA_BLOCK_SIZE	; Buffer2 for stereo audio to be transmitted (at 48kHz)

bufferADC:
bufferADC1	ds	2*DMA_BLOCK_SIZE	; Buffer1 for stereo audio received (at 48kHz)
bufferADC2	ds	2*DMA_BLOCK_SIZE	; Buffer2 for stereo audio received (at 48kHz)


WELCOME		dcb	"DSP Initialized OK.",$A,0 ; Message to be sent to the ColdFire after initialization
	
;-------------------------------------------------------------------------------

	include 'audio_dma.asm'
	include 'host.asm'

;-------------------------------------------------------------------------------

	org	Ll:
	
InputBuffer	ds	BLOCK_SIZE		; Buffer1 for stereo audio processing (at internal sample rate)
OutputBuffer	ds	BLOCK_SIZE		; Buffer2 for stereo audio processing (at internal sample rate)
BypassBuffer	ds	DMA_BLOCK_SIZE		; Buffer to skip sample rate conversion when bypass is enabled

;-------------------------------------------------------------------------------

	org	Pl:

Start:

;-------------------------------------------------------------------------------
; Initialization
;-------------------------------------------------------------------------------
	
	ORI	#$03,MR				; Disable interrupts

	;-----------------------------------------------------------------------
	; Configure Host communications using interrupts
	;-----------------------------------------------------------------------
	
	BSR	ConfigureHostInterrupts

	;-----------------------------------------------------------------------
	; Configure Audio transfers using DMA with double buffering
	;-----------------------------------------------------------------------
	
	BSR	ConfigureAudioDMA

	MOVEC	#$FFFFFF,M7

	ANDI	#<$FC,MR			; Enable interrupts

	MOVE	#WELCOME,R0
	JSR	WriteHostText

;-----------------------------------------------------------------------
; Main Loop
;-----------------------------------------------------------------------

MainLoop:
	
	BSR	ProcessHostData

	BSR	GetInput
	BSR	DoProcessing
	BSR	GenerateOutput

	BSR	WriteHostDspUsage	; Print the usage in the Toolkit window (in debug mode only)
 	BSR	ChangeActiveBufferDMA

	BRA	MainLoop

;------------------------------------------------------------------------------
; DoProcessing
; This routine run all processing code involved. In the example the processing
; is a volume smoothing ramp over the stereo audio data
;------------------------------------------------------------------------------
DoProcessing:

	MOVE	Y:MasterVolumeTarget,A
	MOVE	Y:MasterVolume,X0
	SUB	X0,A
	ASR	#8,A,A
	MOVE	Y:BypassEnabled,Y0
	ADD	X0,A	A,X0

	BRSET	#0,Y0,ProcessBypass

ProcessResampled:

	;-----------------------------------------------------------------------
	; if bypass is not active, process BLOCK_SIZE samples from InputBuffer 
	; to OutputBuffer at modified sample rate (48000 * L / M)
	;-----------------------------------------------------------------------

	; Extra processing code goes here.

	MOVE	#>InputBuffer,R0
	MOVE	#>OutputBuffer,R4

	.LOOP	#BLOCK_SIZE
	
	  MOVE	A,Y0		X:(R0),X1
	  MPYR	X1,Y0,A		Y:(R0)+,Y1
	  MPYR	Y1,Y0,B	
	  MOVE	A,X:(R4)
	  MOVE	Y0,A
	  ADD	X0,A
	  MOVE	B,Y:(R4)+
	
	.ENDL
	
	BRA	ProcessEnd
	
ProcessBypass:
	
	;-----------------------------------------------------------------------
	; if bypass is active, process DMA_BLOCK_SIZE samples from the active 
	; ADC buffer to the active DAC buffer at the Chameleon sample rate (48000 Hz)
	;-----------------------------------------------------------------------

	MOVE	Y:ActiveBufferADC,R0
	MOVE	Y:ActiveBufferDAC,R4
	
	.LOOP	#DMA_BLOCK_SIZE

	  MOVE	A,Y0		X:(R0)+,X1
	  MPYR	X1,Y0,A		X:(R0)+,Y1
	  MPYR	Y1,Y0,B	
	  MOVE	A,X:(R4)+
	  MOVE	Y0,A
	  ADD	X0,A
	  MOVE	B,X:(R4)+

	.ENDL

ProcessEnd:

	MOVE	A,Y:MasterVolume

	RTS

;------------------------------------------------------------------------------

	include 'resample.asm'
	
;------------------------------------------------------------------------------
; GetInput:
;
; This code fills the InputBuffer with contents of the ActiveBufferADC resampled. It
; takes DMA_BLOCK_SIZE samples from the audio input at 48000 Hz and generates 
; BLOCK_SIZE samples at the new working sample rate. If bypass is enabled, no 
; processing is done
;------------------------------------------------------------------------------

GetInput:

	MOVE	Y:BypassEnabledTarget,X0
	MOVE	X0,Y:BypassEnabled
	BRSET	#0,X0,_GetInputEnd

	MOVE	Y:ActiveBufferADC,R0
	MOVE	#>InputBuffer,R4

	JSR	ResampleInput

_GetInputEnd:

	RTS
	
;------------------------------------------------------------------------------
; GenerateOutput:
;
; This code fills ActiveBufferADC with contents of the OutputBuffer resampled. It
; takes BLOCK_SIZE samples from the previously processed buffer at the working 
; sample rate and generates DMA_BLOCK_SIZE samples at 48000 Hz to be sent to the 
; audio output. If bypass is enabled, no processing is done
;------------------------------------------------------------------------------

GenerateOutput:

	MOVE	Y:BypassEnabled,X0
	BRSET	#0,X0,_GenerateOutputEnd

	MOVE	Y:ActiveBufferDAC,R4
	MOVE	#>OutputBuffer,R0
	
	JSR	ResampleOutput

_GenerateOutputEnd:
	
	RTS

;------------------------------------------------------------------------------
	
	end	Start
