;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2003 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

	;include "resample_data.inc"	; already included from main.asm

;------------------------------------------------------------------------------

	org	Xl:

	; Pointers to the delay lines for the polyphase convolutions

pInputDelayL:	dc	fInputDelayL
pInputDelayR:	dc	fInputDelayR

pOutputDelayL:	dc	fOutputDelayL
pOutputDelayR:	dc	fOutputDelayR

;------------------------------------------------------------------------------

	org	Yl:

	; Delay lines for the polyphase convolutions

fInputDelayL:	dsm	kCoeffsInput
fInputDelayR:	dsm	kCoeffsInput

;------------------------------------------------------------------------------

fOutputDelayL:	dsm	kCoeffsOutput
fOutputDelayR:	dsm	kCoeffsOutput


;------------------------------------------------------------------------------
DotProd		macro	Size,Input,Coeffs
;------------------------------------------------------------------------------
;
;	Dot product of 'Size' elements of vectors pointed by 'Input' and 'Coeffs'
;
;	;---------------------------------
;	sum = 0;
;	for (i = 0; i < Size; i++)
;		sum += Input(i) * Coeffs(i)
;	;---------------------------------
;
;	Input: pointer to input vector. It is assumed that the input vector has 'Size'
;	modulo and that Size >= 1
;
;	Coeffs: pointer to coeffs vector
;
;------------------------------------------------------------------------------

	CLR	A

	MOVE	Y:(Input)+,Y0	X:(Coeffs)+,X0

	REP #(Size-1)
	  MAC Y0,X0,A		Y:(Input)+,Y0	X:(Coeffs)+,X0

	MAC Y0,X0,A

	endm

;------------------------------------------------------------------------------

	org	Pl:

;------------------------------------------------------------------------------
; ResampleInput
; Does a L/M ratio resampling of input buffer (size DMA_BLOCK_SIZE) and puts
; the resampled data in the output buffer (size BLOCK_SIZE)
;------------------------------------------------------------------------------
ResampleInput:
;
;	inputs:
;		R0 = pInput	; pointer to stereo audio buffer in external X memory interleaved format
;		R4 = pOutput	; pointer to stereo audio buffer in internal L memory (X -> L, Y -> R)
;------------------------------------------------------------------------------
;
;	register usage:
;
;		R0 = pInput
;		R1 = pCoeffs (for each polyphase filter)
;		R2 = PolySelectorLimits[n][0..1]
;		R3 = PolySelectors
;		R4 = pOutput
; 		R5 = pInputDelayL (circular)
;		R6 = pInputDelayR (circular)
;------------------------------------------------------------------------------
;
;	// algorithm:
;
;	for (int i = 0; i < (DMA_BLOCK_SIZE / kM); i++)
;	{
;		int nLowLimit, nUpLimit, nSelector;
;
;		for (int n = 0; n < kM; n++)
;		{
;			nLowLimit = fPolyselectorsInputLimits[n][0];
;			nUpLimit = fPolyselectorsInputLimits[n][1];
;
;			*pInputDelayL = *pInput++;
;			*pInputDelayR = *pInput++;
;
;			for (int j = nLowLimit; j <= nUpLimit; j++)
;			{
;				nSelector = fPolyselectorsInput[j];
;				*pOutputL++ = DotProd(pInputDelayL, fResampleInputCoeffs[nSelector], kCoeffsInput);
;				*pOutputR++ = DotProd(pInputDelayR, fResampleInputCoeffs[nSelector], kCoeffsInput);
;			}
;
;			pInputDelayL--;
;			pInputDelayR--;
;		}
;
;	}
;------------------------------------------------------------------------------

	MOVEC   #(kCoeffsInput-1),M5
	MOVE    X:pInputDelayL,R5

	MOVEC   #(kCoeffsInput-1),M6
	MOVE    X:pInputDelayR,R6

;	for (i = 0; i < (DMA_BLOCK_SIZE / kM); i++)

	.LOOP	#(DMA_BLOCK_SIZE/kM)

;	{
;		int nLowLimit, nUpLimit, nSelector;

		;-----------------------------------------------------------------------------
		; for a fixed resampling ratio, the following two for loops can be unrolled for
		; greater optimization, since the sequence of the polyphase filter index for each
		; product is known, so avoiding the use of the polyphase selectors
		; and limits.
		;-----------------------------------------------------------------------------

;		for (n = 0; n < kM; n++)

		MOVE    #fPolyselectorsInputLimits,R2

;		{
		.LOOP	#kM

;			nLowLimit = fPolyselectorsInputLimits[n][0];
;			nUpLimit = fPolyselectorsInputLimits[n][1];

			MOVE    #fPolyselectorsInput,R3
			MOVE    X:(R2)+,N3		; nLowLimit
			MOVE    X:(R2)+,N2		; nUpLimit

;			*pInputDelayL = *pInput++;
;			*pInputDelayR = *pInput++;

			MOVE    X:(R0)+,X0
			MOVE    X0,Y:(R5)

			MOVE    X:(R0)+,X0
			MOVE    X0,Y:(R6)

			MOVE    N3,B
			MOVE    (R3)+N3			; &fPolyselectorsInput[nLowLimit]

;			for (j = nLowLimit; j <= nUpLimit; j++)

			BRA     >BeginConvolveLoopInput

;			{
;				nSelector = fPolyselectorsInput[j];

ConvolveLoopInput:

				MOVE    X:(R3)+,X0
				MPYI    #kCoeffsInput,X0,A
				ASR	A	#fResampleInputCoeffs,B0
				ADD	A,B

				NOP

				MOVE    B0,R1		; &fResampleInputCoeffs[nSelector]
				MOVE    B0,N4

				NOP

;				*pOutputL++ = ASM_DotProd(pInputDelayL, fResampleInputCoeffs[nSelector], kCoeffsInput);

				DotProd	kCoeffsInput,R5,R1

				MOVE    N4,R1
				MOVE    A,X:(R4)

				NOP

;				*pOutputR++ = DotProd(pInputDelayR, fResampleInputCoeffs[nSelector], kCoeffsInput);

				DotProd	kCoeffsInput,R6,R1

				MOVE    N0,B
				ADD     #1,B

				MOVE    A,Y:(R4)+

BeginConvolveLoopInput:
				MOVE    N2,X0
				CMP     X0,B
				MOVE    B1,N0
				BLE     >ConvolveLoopInput
;			}

;			pInputDelayL--;
;			pInputDelayR--;

			MOVE    (R5)-
			MOVE    (R6)-

		.ENDL
		NOP
	.ENDL

;		}
;	}
;}

	MOVE    R5,X:pInputDelayL
	MOVE    R6,X:pInputDelayR

	MOVEC	M7,M5
	MOVEC	M7,M6

	RTS

;------------------------------------------------------------------------------
; ResampleOutput
; Does a M/L ratio resampling of input buffer (size BLOCK_SIZE) and puts
; the resampled data in the output buffer (size DMA_BLOCK_SIZE)
;------------------------------------------------------------------------------
ResampleOutput:
;
;	inputs:
;		R0 = pInput	; pointer to stereo audio buffer in internal L memory (X -> L, Y -> R)
;		R4 = pOutput	; pointer to stereo audio buffer in external X memory interleaved format
;------------------------------------------------------------------------------
;
;	register usage:
;
;		R0 = pInput
;		R1 = pCoeffs (for each polyphase filter)
;		R2 = PolySelectorLimits[n][0..1]
;		R3 = PolySelectors
;		R4 = pOutput
; 		R5 = pInputDelayL (circular)
;		R6 = pInputDelayR (circular)
;------------------------------------------------------------------------------
;
;	// algorithm
;
;	for (int i = 0; i < (BLOCK_SIZE / kL); i++)
;	{
;		for (int n = 0; n < kL; n++)
;		{
;			nLowLimit = fPolyselectorsOutputLimits[n][0];
;			nUpLimit = fPolyselectorsOutputLimits[n][1];
;			nSelector;
;
;			*pOutputDelayL = *pInputL++;
;			*pOutputDelayR = *pInputR++;
;
;			for (int j = nLowLimit; j <= nUpLimit; j++)
;			{
;				nSelector = fPolyselectorsUp[j];
;				*pOutput++ = DotProd(pOutputDelayL, fResampleOutputCoeffs[nSelector], kCoeffsOutput);
;				*pOutput++ = DotProd(pOutputDelayR, fResampleOutputCoeffs[nSelector], kCoeffsOutput);
;			}
;
;			pOutputDelayL--;
;			pOutputDelayR--;
;		}
;	}
;------------------------------------------------------------------------------

	MOVEC   #(kCoeffsOutput-1),M5
	MOVE    X:pOutputDelayL,R5

	MOVEC   #(kCoeffsOutput-1),M6
	MOVE    X:pOutputDelayR,R6

;	for (i = 0; i < (BLOCK_SIZE / kL); i++)

	.LOOP	#(BLOCK_SIZE/kL)

;	{
;		int nLowLimit, nUpLimit, nSelector;

		;-----------------------------------------------------------------------------
		; for a fixed resampling ratio, the following two for loops can be unrolled for
		; greater optimization, since the sequence of the polyphase filter index for each
		; product is known, so avoiding the use of the polyphase selectors
		; and limits.
		;-----------------------------------------------------------------------------

;		for (n = 0; n < kL; n++)

		MOVE    #fPolyselectorsOutputLimits,R2
;		{
		.LOOP	#kL

;			nLowLimit = fPolyselectorsOutputLimits[n][0];
;			nUpLimit = fPolyselectorsOutputLimits[n][1];

			MOVE    #fPolyselectorsOutput,R3
			MOVE    X:(R2)+,N3		; nLowLimit
			MOVE    X:(R2)+,N2		; nUpLimit

;			*pOutputDelayL = *pInputL++;
;			*pOutputDelayR = *pInputR++;

			MOVE    X:(R0),X0
			MOVE    X0,Y:(R5)
			MOVE    Y:(R0)+,X0
			MOVE    X0,Y:(R6)

			MOVE    N3,B
			MOVE    (R3)+N3			; &fPolyselectorsOutput[nLowLimit]

;			for (j = nLowLimit; j <= nUpLimit; j++)

			BRA     >BeginConvolveLoopOutput

;			{
;				nSelector = fPolyselectorsOutput[j];

ConvolveLoopOutput:

				MOVE    X:(R3)+,X0
				MPYI    #>kCoeffsOutput,X0,A
				ASR	A	#>fResampleOutputCoeffs,B0
				ADD	A,B

				NOP
				MOVE    B0,R1		; &fResampleOutputCoeffs[nSelector]
				MOVE    B0,N4
				NOP

;				*pOutput++ = DotProd(pOutputDelayL, fResampleOutputCoeffs[nSelector], kCoeffsOutput);

				DotProd	kCoeffsOutput,R5,R1

				MOVE    N4,R1
				MOVE	A,X:(R4)+
				NOP

;				*pOutputR++ = DotProd(pOutputDelayR, fResampleOutputCoeffs[nSelector], kCoeffsOutput);

				DotProd	kCoeffsOutput,R6,R1

				MOVE    N0,B
				ADD     #1,B

				MOVE	A,X:(R4)+

BeginConvolveLoopOutput:
				MOVE    N2,X0
				CMP     X0,B
				MOVE    B1,N0
				BLE     >ConvolveLoopOutput
;			}

;			pOutputDelayL--;
;			pOutputDelayR--;

			MOVE    (R5)-
			MOVE    (R6)-

		.ENDL
		NOP
	.ENDL

;		}
;	}
;}

	MOVE    R5,X:pOutputDelayL
	MOVE    R6,X:pOutputDelayR

	MOVEC	M7,M5
	MOVEC	M7,M6

	RTS

;------------------------------------------------------------------------------