//******************************************************************************
// C H A M E L E O N    S. D. K.                                               *
//******************************************************************************
//  $Archive:: /Chameleon.sdk/src/examples/resample/scilab/printdata.sci       $
//     $Date::                                                                 $
// $Revision:: 1                                                               $
//-----------------------------------------------------------------------------*
// This file is part of the Chameleon Software Development Kit                 *
//                                                                             *
// Copyright (C) 2001-2003 Soundart                                            *
// www.soundart-hot.com                                                        *
// support@soundart-hot.com                                                    *
//*****************************************************************************/

///////////////////////////////////////////////////////////////////////////////

function printdata(L, M, NCoeffs)

// function printdata(L, M, [, NCoeffs])
//
// generates the file 'resample_data.inc' to be used in the DSP. It prints all the
// data needed by the algorithm: the coefficients of the filters and polyphase selectors
// for the resampling of the input at ratio L/M and the output at ratio M/L
//
// parameters:
// 	L : Interpolation ratio for the input, decimation ratio for the output (must be integer)
// 	M : Decimation ratio for the input, interpolation ratio for the output (must be integer)
//	NCoeffs: number of coefficients of the Kaiser Low Pass filters. If is not specified,
//	an optimum number is computed. Both filters use the same number of coefficients

	getf('resfilt.sci');

	if (argn(2) == 2)
		NCoeffs = -1;
	else
		if (argn(2) ~= 3)
			error('Wrong number of input args');
		end;
	end;

	At = 80; // Default filter stopband attenuation set to 80 dB

	h = resfilt(L, M, At, NCoeffs);	// Get the resample filter for the input data (48000 Hz to f')
	hp_input = getpolyphase(L, h);	// Transform the input resample filter into its polyphase form
	CoeffsInput = length(hp_input(:,1)); // Get each subfilter length
	[sel_input, lim_input] = getpolyselector(L, M);	// Get the polyphase selectors and loop limits

	h = resfilt(M, L, At, NCoeffs);	// Get the resample filter for the output data (f' to 48000 Hz)
	hp_output = getpolyphase(M, h);	// Transform the upsample filter into its polyphase form
	CoeffsOutput = length(hp_output(:,1)); // Get each subfilter length
	[sel_output, lim_output] = getpolyselector(M, L); // Get the polyphase selectors and loop limits

	// print the data in the DSP format

	f = mopen('resample_data.inc', 'w');

	fprintf(f, ';*******************************************************************************\n');
	fprintf(f, '; C H A M E L E O N   DSP Assembler file                                       *\n');
	fprintf(f, ';*******************************************************************************\n');
	fprintf(f, '; Copyright (C) 2001-2003 Soundart                                             *\n');
	fprintf(f, ';*******************************************************************************\n');
	fprintf(f, '; www.soundart-hot.com                                                         *\n');
	fprintf(f, '; support@soundart-hot.com                                                     *\n');
	fprintf(f, ';*******************************************************************************\n');

	fprintf(f, '\n');
	fprintf(f, '\topt NOW\n');

	fprintf(f, '\n');
	fprintf(f, ';------------------------------------------------------------------------------\n');
	fprintf(f, '\n');

	fprintf(f, 'kL\t\tequ\t%d\n', L);
	fprintf(f, 'kM\t\tequ\t%d\n', M);
	fprintf(f, '\n');
	fprintf(f, 'kCoeffsInput\tequ\t%d\n', CoeffsInput);
	fprintf(f, 'kCoeffsOutput\tequ\t%d\n', CoeffsOutput);

	fprintf(f, '\n');
	fprintf(f, ';------------------------------------------------------------------------------\n');
	fprintf(f, '\n');

	fprintf(f, '\torg Xl:\n');
	fprintf(f, '\n');

	fprintf(f, 'fPolyselectorsInput:\n');
	fprintf(f, '\n');
	fprintf(f, '\tdc\t$%06X\n', sel_input);
	fprintf(f, '\n');

	fprintf(f, 'fPolyselectorsInputLimits:\n');
	fprintf(f, '\n');
	for i=1:M,
		fprintf(f, '\tdc\t$%06X\n', lim_input(i,:)');
		fprintf(f, '\n');
	end;

	fprintf(f, 'fPolyselectorsOutput:\n');
	fprintf(f, '\n');
	fprintf(f, '\tdc\t$%06X\n', sel_output);
	fprintf(f, '\n');

	fprintf(f, 'fPolyselectorsOutputLimits:\n');
	fprintf(f, '\n');

	for i=1:L,
		fprintf(f, '\tdc\t$%06X\n', lim_output(i,:)');
		fprintf(f, '\n');
	end;

	fprintf(f, ';------------------------------------------------------------------------------\n');
	fprintf(f, '\n');

	fprintf(f, 'fResampleInputCoeffs:\n');
	fprintf(f, '\n');

	for i=1:L,
		fprintf(f, '\tdc\t%.22f\n', hp_input(:,i));
		fprintf(f, '\n');
	end;

	fprintf(f, ';------------------------------------------------------------------------------\n');
	fprintf(f, '\n');

	fprintf(f, 'fResampleOutputCoeffs:\n');
	fprintf(f, '\n');

	for i=1:M,
		fprintf(f, '\tdc\t%.22f\n', hp_output(:,i));
		fprintf(f, '\n');
	end;

	fprintf(f, ';------------------------------------------------------------------------------\n');
	fprintf(f, '\n');
	fprintf(f, '\topt\tW\n');
	mclose(f);

endfunction
