//******************************************************************************
// C H A M E L E O N    S. D. K.                                               *
//******************************************************************************
//  $Archive:: /Chameleon.sdk/src/examples/resample/scilab/resample.sci        $
//     $Date::                                                                 $
// $Revision:: 1                                                               $
//-----------------------------------------------------------------------------*
// This file is part of the Chameleon Software Development Kit                 *
//                                                                             *
// Copyright (C) 2001-2003 Soundart                                            *
// www.soundart-hot.com                                                        *
// support@soundart-hot.com                                                    *
//*****************************************************************************/

///////////////////////////////////////////////////////////////////////////////

function resample(L, M, filenamewav, NCoeffs)

// function resample(L, M, filenamewav [, NCoeffs])
//
// performs a L/M ratio resample over the WAV file specified. The resampled
// audio data is stored in a WAV file called 'out.wav'
//
// parameters:
// 	L : Interpolation ratio (must be integer)
// 	M : Decimation ratio (must be integer)
// 	filenamewav: string with the name of the WAV file to be processed
//		     (whithout extension)
//	NCoeffs: number of coefficients of the Kaiser Low Pass filter. If is not
//		 specified, an optimum number is computed.

	getf('resfilt.sci');

	filenamewav = filenamewav + '.wav';
	[x,fs,bits] = wavread(filenamewav);	// get the input signal in x
	wavlen = length(x);

	Ir = L; 		// interpolation ratio
	Dr = M;			// downsample ratio

	// Get the LowPass filter

	if (argn(2) == 4)
		h = resfilt(Ir, Dr, 80, NCoeffs);
	else
		if (argn(2) == 3)
			h = resfilt(Ir, Dr, 80);
		else
			if (argn(2) ~= 4)
				error('Wrong number of input args');
			end;
		end;
	end;

	hp = getpolyphase(Ir, h);	// Convert the LowPass filter to its polyphase form

	N2 = length(hp(:,1)); 	// length of each polyphase filter

	w = zeros(N2, 1);	// delay line for the polyphase convolution

	// N = 2*Ir*K + 1	// num of coeffs
	// N = 2*N2 + 1
	K = (2*N2)/(2*Ir);	// filter delay

	// init the delay line
	for i=1:K
		if ((1 + (K + 1) - i) < N2)
			w(1 + (K + 1) - i) = x(i);
		end;
	end;

	y = zeros(wavlen * Ir/Dr, 1);

	inbuf = zeros(Dr,1);
	outbuf = zeros(Ir,1);

	[sel, lim] = getpolyselector(L, M); // precompute polyphase selectors and their limits for
					    // the inner loop in the convolution

	sel = sel + 1; // adapt to scilab vector index (1 based)
	lim = lim + 1; // adapt to scilab vector index (1 based)

	// do the polyphase convolution

	for i = 1:((wavlen - K) / Dr),

		inbuf(1:Dr) = x((Dr*i - (Dr - 1)) + K : (Dr*i - (Dr - 1)) + (Dr - 1) + K);

		for n = 1:Dr

			// feed the delay line with the next sample
			w(1) = inbuf(n);

			// polyphase products arranged to generate M output samples from L input sample
			// according to the polyphase selectors

			for j = lim(n,1):lim(n,2)
				outbuf(j) = sum(hp(:,sel(j)) .* w);
			end;

			// advance the delay line
			w = delay(w);
		end;

		y((Ir*i - (Ir - 1)) : (Ir*i - (Ir - 1)) + (Ir - 1)) = outbuf(1:Ir);
	end;

	wavwrite(y,fs*Ir/Dr,bits,'out.wav');

endfunction

///////////////////////////////////////////////////////////////////////////////

function wd = delay(w)

	D = length(w);
	wd = zeros(D,1);
	wd(2:D) = w(1:(D-1));

endfunction

///////////////////////////////////////////////////////////////////////////////